@@ -10,13 +10,14 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
-# pylint: disable=import-private-name
+# pylint: disable=import-private-name, too-many-lines
 
 """The website of the AN. Loads config and modules and starts Tornado."""
 
 from __future__ import annotations
 
 import asyncio
+import contextlib
 import importlib
 import logging
 import os
@@ -24,6 +25,8 @@ import platform
 import signal
 import ssl
 import sys
+import threading
+import time
 import types
 from asyncio import AbstractEventLoop
 from asyncio.runners import _cancel_all_tasks  # type: ignore[attr-defined]
@@ -97,6 +100,8 @@ CONFIG.read("config.ini", encoding="UTF-8")
 
 LOGGER: Final = logging.getLogger(__name__)
 
+HEARTBEAT: float = 0
+
 
 # add all the information from the packages to a list
 # this calls the get_module_info function in every file
@@ -121,9 +126,9 @@ def get_module_infos() -> str | tuple[ModuleInfo, ...]:
         if _module_infos:
             module_infos.extend(_module_infos)
             loaded_modules.append(potential_module)
-            LOGGER.info(
+            LOGGER.debug(
                 "Found module_infos in %s.__init__.py, "
-                "do not search in other files in the module.",
+                "not searching in other modules in the package.",
                 potential_module,
             )
             continue
@@ -334,7 +339,7 @@ def make_app(config: ConfigParser) -> str | Application:
         HANDLERS=handlers,
         # General settings
         autoreload=False,
-        debug=bool(sys.flags.dev_mode),
+        debug=sys.flags.dev_mode,
         default_handler_class=NotFoundHandler,
         compress_response=config.getboolean(
             "GENERAL", "COMPRESS_RESPONSE", fallback=False
@@ -651,7 +656,7 @@ def setup_elasticsearch(app: Application) -> None | AsyncElasticsearch:
 async def check_elasticsearch(app: Application) -> None:  # pragma: no cover
     """Check Elasticsearch."""
     # pylint: disable=invalid-name
-    while True:  # pylint: disable=while-used
+    while not EVENT_SHUTDOWN.is_set():  # pylint: disable=while-used
         es: AsyncElasticsearch = cast(
             AsyncElasticsearch, app.settings.get("ELASTICSEARCH")
         )
@@ -681,8 +686,8 @@ async def setup_elasticsearch_configs(  # noqa: C901
 ) -> None:
     """Setup Elasticsearch configs."""  # noqa: D401
     # pylint: disable=too-complex, too-many-branches
-    get: Callable[..., Coroutine[Any, Any, dict[str, Any]]]
-    put: Callable[..., Coroutine[Any, Any, dict[str, Any]]]
+    get: Callable[..., Coroutine[None, None, dict[str, Any]]]
+    put: Callable[..., Coroutine[None, None, dict[str, Any]]]
     for i in range(3):
         if i == 0:  # pylint: disable=compare-to-zero
             what = "ingest_pipelines"
@@ -794,7 +799,7 @@ def setup_redis(app: Application) -> None | Redis[str]:
 
 async def check_redis(app: Application) -> None:  # pragma: no cover
     """Check Redis."""
-    while True:  # pylint: disable=while-used
+    while not EVENT_SHUTDOWN.is_set():  # pylint: disable=while-used
         redis: Redis[str] = cast("Redis[str]", app.settings.get("REDIS"))
         try:
             await redis.ping()
@@ -832,13 +837,31 @@ def install_signal_handler() -> None:  # pragma: no cover
         signal.signal(signal.SIGHUP, signal_handler)
 
 
+async def heartbeat() -> None:
+    """Heartbeat."""
+    global HEARTBEAT  # pylint: disable=global-statement
+    while True:  # pylint: disable=while-used
+        HEARTBEAT = time.monotonic()
+        await asyncio.sleep(1)
+
+
+def supervise(loop: AbstractEventLoop) -> None:
+    """Supervise."""
+    while not loop.is_closed():  # pylint: disable=while-used
+        if time.monotonic() - HEARTBEAT >= 10:
+            with contextlib.suppress(OSError):
+                os.kill(os.getpid(), getattr(signal, "CTRL_BREAK_EVENT", 9))
+        time.sleep(1)
+
+
 def main() -> None | int | str:  # noqa: C901  # pragma: no cover
     """
     Start everything.
 
     This is the main function that is called when running this file.
     """
-    # pylint: disable=too-complex, too-many-branches, too-many-statements
+    # pylint: disable=too-complex, too-many-branches
+    # pylint: disable=too-many-locals, too-many-statements
     setproctitle(NAME)
     install_signal_handler()
     setup_logging(CONFIG)
@@ -847,8 +870,7 @@ def main() -> None | int | str:  # noqa: C901  # pragma: no cover
 
     if platform.system() == "Windows":
         LOGGER.warning(
-            "Please note that running %s on Windows is not officially"
-            " supported",
+            "Running %s on Windows is not officially supported",
             NAME.removesuffix("-dev"),
         )
 
@@ -894,7 +916,7 @@ def main() -> None | int | str:  # noqa: C901  # pragma: no cover
     )
 
     if unix_socket_path:
-        os.makedirs(unix_socket_path, exist_ok=True)
+        os.makedirs(unix_socket_path, 0o755, True)
         sockets.append(
             tornado.netutil.bind_unix_socket(
                 os.path.join(unix_socket_path, f"{NAME}.sock"),
@@ -905,25 +927,28 @@ def main() -> None | int | str:  # noqa: C901  # pragma: no cover
     processes = CONFIG.getint(
         "GENERAL",
         "PROCESSES",
-        fallback=0
+        fallback=2
         if sys.flags.dev_mode
         else (tornado.process.cpu_count() if hasattr(os, "fork") else 0),
     )
 
+    task_id: int | None = None
+
     if processes > 0:
         setproctitle(f"{NAME} - Master")
-        tornado.process.fork_processes(processes)
+
+        task_id = tornado.process.fork_processes(processes)
+
+        setproctitle(f"{NAME} - Worker {task_id}")
+
         # yeet all children (there should be none, but do it regardless, just in case)
         process._children.clear()  # type: ignore[attr-defined]  # pylint: disable=protected-access  # noqa: B950
+
         del AUTHORS_CACHE.control.created_by_ultra  # type: ignore[attr-defined]
         del QUOTES_CACHE.control.created_by_ultra  # type: ignore[attr-defined]
         del WRONG_QUOTES_CACHE.control.created_by_ultra  # type: ignore[attr-defined]
         del geoip.__kwdefaults__["caches"].control.created_by_ultra
 
-    task_id = tornado.process.task_id()
-
-    if task_id is not None:
-        setproctitle(f"{NAME} - Worker {task_id}")
         if unix_socket_path:
             sockets.append(
                 tornado.netutil.bind_unix_socket(
@@ -934,16 +959,17 @@ def main() -> None | int | str:  # noqa: C901  # pragma: no cover
 
     # get loop after forking
     loop = asyncio.get_event_loop_policy().get_event_loop()
+    setup_webhook_logging(CONFIG, loop)
 
     _ = asyncio.get_event_loop
     asyncio.get_event_loop = lambda: loop
     server.add_sockets(sockets)
     asyncio.get_event_loop = _
 
-    setup_webhook_logging(CONFIG, loop)
-
     # pylint: disable=unused-variable
 
+    heartbeat_task = loop.create_task(heartbeat())  # noqa: F841
+
     wait_for_shutdown_task = loop.create_task(wait_for_shutdown())  # noqa: F841
 
     if CONFIG.getboolean("ELASTICSEARCH", "ENABLED", fallback=False):
@@ -952,22 +978,41 @@ def main() -> None | int | str:  # noqa: C901  # pragma: no cover
     if CONFIG.getboolean("REDIS", "ENABLED", fallback=False):
         check_redis_task = loop.create_task(check_redis(app))  # noqa: F841
 
-    if not task_id:
+    if not task_id:  # update from one process only
         quotes_cache_update_task = loop.create_task(  # noqa: F841
             update_cache_periodically(app)
         )
 
+    # pylint: enable=unused-variable
+
+    if CONFIG.getboolean("GENERAL", "SUPERVISE", fallback=False):
+        global HEARTBEAT  # pylint: disable=global-statement
+        HEARTBEAT = time.monotonic()
+        threading.Thread(
+            target=supervise, name="supervisor", args=(loop,), daemon=True
+        ).start()
+
     try:
         loop.run_forever()
+        EVENT_SHUTDOWN.set()
     finally:
         try:
             server.stop()
             loop.run_until_complete(asyncio.sleep(1))
-            loop.run_until_complete(server.close_all_connections())
+            loop.run_until_complete(
+                asyncio.wait_for(server.close_all_connections(), 5)
+            )
+
             if redis := app.settings.get("REDIS"):
-                loop.run_until_complete(redis.close(close_connection_pool=True))
+                loop.run_until_complete(
+                    asyncio.wait_for(redis.close(close_connection_pool=True), 5)
+                )
+
             if elasticsearch := app.settings.get("ELASTICSEARCH"):
-                loop.run_until_complete(elasticsearch.close())
+                loop.run_until_complete(
+                    asyncio.wait_for(elasticsearch.close(), 5)
+                )
+
         finally:
             try:
                 _cancel_all_tasks(loop)
